MODULE Display;
(* =========================================================================
   Example ARM Oberon Module

   Description:
     Write line-oriented output to an LCD display

   Target:
     Dependent on the imported module: LCD

   Tested on:
     Olimex LPC1766-STK Development Prototype Board

   (c) 2011-2012 CFB Software
   http://www.astrobe.com

   ========================================================================= *)

IMPORT LCD := LCDEpson, Fonts;

CONST
  LF = 0AX;

TYPE
  Row = ARRAY LCD.MaxCol OF CHAR;
  Screen = ARRAY LCD.MaxRow OF Row;

VAR
  rowNo, colNo: INTEGER;
  screen: Screen;
  blankRow: Row;

PROCEDURE WriteChar(ch: CHAR; r, c: INTEGER);
BEGIN
  LCD.DrawChar(LCD.White, ch, c * 8, r * 10 + 2)
END WriteChar;


PROCEDURE WriteRow(row: Row);
VAR
  c: INTEGER;
BEGIN
  FOR c := 0 TO LCD.MaxCol - 1 DO
    WriteChar(row[c], rowNo, c)
  END
END WriteRow;


PROCEDURE ScrollUp;
VAR
  r: INTEGER;
BEGIN
  (* Move rows up *)
  FOR r := 1 TO LCD.MaxRow - 1 DO
    rowNo := r - 1;
    screen[rowNo] := screen[r];
    WriteRow(screen[rowNo])
  END;
  rowNo := LCD.MaxRow - 1;
  colNo := 0;
  screen[rowNo] := blankRow
END ScrollUp;


PROCEDURE Ln*();
BEGIN
  IF rowNo < LCD.MaxRow - 1 THEN
    WriteRow(screen[rowNo]);
    INC(rowNo);
    colNo := 0;
    screen[rowNo] := blankRow
  ELSE
    ScrollUp
  END
END Ln;


PROCEDURE Char*(ch: CHAR);
BEGIN
  IF ch = LF THEN
    Ln
  ELSIF ch >= " " THEN
    screen[rowNo, colNo] := ch;
    INC(colNo);
    IF colNo = LCD.MaxCol THEN Ln END
  END
END Char;


PROCEDURE String*(s: ARRAY OF CHAR);
VAR
  i: INTEGER;
BEGIN
  i := 0;
  WHILE (i < LEN(s)) & (s[i] # 0X) DO Char(s[i]); INC(i) END
END String;


PROCEDURE Init*();
BEGIN
  LCD.Init();
  ASSERT(LCD.LoadFont("Fonts"), 100);
  LCD.ClearScreen(LCD.Black)
END Init;

BEGIN
  (* Initialise the memory representation of the screen *)
  FOR colNo := 0 TO LCD.MaxCol - 1 DO blankRow[colNo] := " " END;
  FOR rowNo := 0 TO LCD.MaxRow - 1 DO screen[rowNo] := blankRow END;
  rowNo := 0;
  colNo := 0
END Display.
